package com.cultural.heritage.service.wxpay.impl;


import cn.binarywang.wx.miniapp.api.WxMaMsgService;
import cn.binarywang.wx.miniapp.api.WxMaService;
import cn.binarywang.wx.miniapp.bean.WxMaSubscribeMessage;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.config.WxOpenConfig;
import com.cultural.heritage.config.WxPayConfig;
import com.cultural.heritage.constant.OrderStatusConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.entity.Order;
import com.cultural.heritage.service.order.OrderService;
import com.cultural.heritage.service.wxpay.WeChatService;
import com.wechat.pay.java.core.notification.NotificationParser;
import com.wechat.pay.java.core.notification.RequestParam;
import com.wechat.pay.java.service.payments.jsapi.model.Amount;
import com.wechat.pay.java.service.payments.jsapi.model.Payer;
import com.wechat.pay.java.service.payments.jsapi.model.PrepayRequest;
import com.wechat.pay.java.service.payments.jsapi.model.PrepayWithRequestPaymentResponse;
import com.wechat.pay.java.service.payments.model.Transaction;
import com.wechat.pay.java.service.refund.model.AmountReq;
import com.wechat.pay.java.service.refund.model.CreateRequest;
import com.wechat.pay.java.service.refund.model.Refund;
import com.wechat.pay.java.service.refund.model.RefundNotification;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.BufferedReader;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * @author 玄德
 */
@Slf4j
@Service
public class WeChatServiceImpl implements WeChatService {

    @Resource
    private WxPayConfig wxPayConfig;

    @Resource
    private WxOpenConfig wxOpenConfig;

    @Resource
    private OrderService orderService;


    /**
     * 请求参数
     */
    public static RequestParam requestParam = null;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public synchronized PrepayWithRequestPaymentResponse createPayment(String orderId, String miniOpenId, BigDecimal amount) {
        // request.setXxx(val)设置所需参数，具体参数可见Request定义
        PrepayRequest request = new PrepayRequest();
        // 金额
        Amount WxAmount = new Amount();
        WxAmount.setTotal(amount.movePointRight(2).intValue());
        WxAmount.setCurrency("CNY");
        request.setAmount(WxAmount);
        // 公众号id
        request.setAppid(wxPayConfig.getAppId());
        // 商户号
        request.setMchid(wxPayConfig.getMerchantId());
        // 支付者信息
        Payer payer = new Payer();
        payer.setOpenid(miniOpenId);
        request.setPayer(payer);
        // 获取订单号
        Order order = orderService.getById(orderId);
        String orderNumber = order.getOrderNumber();
        // 描述
        request.setDescription("订单号：" + orderNumber);
        // 微信回调地址
        request.setNotifyUrl(wxPayConfig.getNotifyUrl() + "/api/wechat/payment/callback");
        // 商户订单号

        request.setOutTradeNo(orderNumber);
        //返回数据，前端调起支付
        return wxPayConfig.getJsapiServiceExtension().prepayWithRequestPayment(request);
    }

    @Override
    public Transaction getTransactionInfo(HttpServletRequest request) {
        NotificationParser notificationParser = getNotificationParser(request);
        return notificationParser.parse(requestParam, Transaction.class);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public synchronized boolean paymentCallback(Transaction transaction) {
        System.out.println("---------------------------微信支付回调（开始）-------------------------------");
        // 获取订单信息
        String orderIdByString = transaction.getOutTradeNo();
        QueryWrapper<Order> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("orderNumber", orderIdByString);
        Order order = orderService.getOne(queryWrapper);
        if (order == null) {
            log.error("订单不存在");
            throw new BusinessException(ErrorCode.NOT_FOUND_ERROR, "订单不存在，订单号：" + transaction.getOutTradeNo());
        }
        // 修改订单状态
        order.setOrderStatus(OrderStatusConstant.PENDING_SHIPMENT);
        boolean update = orderService.updateById(order);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "订单状态修改失败");

        System.out.println("---------------------------微信支付回调（结束）-------------------------------");
        return true;
    }

    @Override
    public Refund refundPayment(String orderId, BigDecimal amount) {
        // 获取订单号
        Order order = orderService.getById(orderId);
        ThrowUtils.throwIf(order == null, ErrorCode.OPERATION_ERROR, "该订单不存在");
        String orderNumber = order.getOrderNumber();
        // 退款请求
        CreateRequest createRequest = new CreateRequest();
        // 商户订单号
        createRequest.setOutTradeNo(orderNumber);
        // 商户退款单号
        createRequest.setOutRefundNo(orderNumber);
        // 退款结果回调
        createRequest.setNotifyUrl(wxPayConfig.getNotifyUrl() + "/api/wechat/refund/callback");
        // 退款金额
        AmountReq amountReq = new AmountReq();
        long refundAmount = amount.movePointRight(2).intValue();
        amountReq.setRefund(refundAmount);
        amountReq.setTotal(refundAmount);
        amountReq.setCurrency("CNY");
        createRequest.setAmount(amountReq);
        // 申请退款
        System.out.println("退款请求：" + createRequest);
        Refund refund = wxPayConfig.getRefundService().create(createRequest);
        System.out.println("退款申请结果：" + refund);
        return refund;
    }

    @Override
    public RefundNotification getRefundInfo(HttpServletRequest request) {
        NotificationParser notificationParser = getNotificationParser(request);
        return notificationParser.parse(requestParam, RefundNotification.class);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public synchronized boolean refundCallback(RefundNotification refundNotification) {
        System.out.println("---------------------------微信退款回调（开始）-------------------------------");
        // 获取订单信息
        String orderIdByString = refundNotification.getOutTradeNo();
        QueryWrapper<Order> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("orderNumber", orderIdByString);
        Order order = orderService.getOne(queryWrapper);
        if (order == null) {
            log.error("订单不存在");
            throw new BusinessException(ErrorCode.NOT_FOUND_ERROR, "订单不存在，订单号：" + orderIdByString);
        }

        // 修改订单状态
        order.setOrderStatus(OrderStatusConstant.PAYMENT_REFUNDED);
        boolean update = orderService.updateById(order);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "订单状态修改失败");
        System.out.println("---------------------------微信退款回调（结束）-------------------------------");
        return true;
    }

    @Override
    public boolean sendSubscribeMessage(String openid, String templateId) {
        try {
            // 获取小程序对象
            WxMaService wxMaService = wxOpenConfig.getWxMaService();
            // 获取消息接口
            WxMaMsgService msgService = wxMaService.getMsgService();
            // 构建订阅消息体
            WxMaSubscribeMessage wxMaSubscribeMessage = new WxMaSubscribeMessage();
            wxMaSubscribeMessage.setToUser(openid);
            wxMaSubscribeMessage.setTemplateId(templateId);
            List<WxMaSubscribeMessage.MsgData> msgDataList = getMsgData();
            wxMaSubscribeMessage.setData(msgDataList);
            // 发送消息
            msgService.sendSubscribeMsg(wxMaSubscribeMessage);
            return true;
        } catch (Exception e) {
            log.error("发送订阅消息失败：", e);
            throw new BusinessException(ErrorCode.SYSTEM_ERROR, "发送订阅消息失败");
        }
    }

    /**
     * 提取消息数据
     */
    private static List<WxMaSubscribeMessage.MsgData> getMsgData() {
        List<WxMaSubscribeMessage.MsgData> msgDataList = new ArrayList<>();
        String[][] dataItems = {
                {"character_string1", "2022060215050983711"},
                {"time2", "2022-06-22 15:48:05"},
                {"thing3", "测试店"},
                {"amount4", "99999元"},
                {"thing5", "收款到账，请查看"}
        };
        for (String[] item : dataItems) {
            WxMaSubscribeMessage.MsgData msgData = new WxMaSubscribeMessage.MsgData();
            msgData.setName(item[0]);
            msgData.setValue(item[1]);
            msgDataList.add(msgData);
        }
        return msgDataList;
    }

    /**
     * 根据微信官方发送的请求获取信息
     */
    @SneakyThrows
    public NotificationParser getNotificationParser(HttpServletRequest request) {
        System.out.println("---------------------------获取信息-------------------------------");
        // 获取RSA配置
        NotificationParser notificationParser = new NotificationParser(wxPayConfig.getRSAConfig());
        // 构建请求
        StringBuilder bodyBuilder = new StringBuilder();
        BufferedReader reader = request.getReader();
        String line;
        while ((line = reader.readLine()) != null) {
            bodyBuilder.append(line);
        }
        String body = bodyBuilder.toString();
        String timestamp = request.getHeader("Wechatpay-Timestamp");
        String nonce = request.getHeader("Wechatpay-Nonce");
        String signature = request.getHeader("Wechatpay-Signature");
        String singType = request.getHeader("Wechatpay-Signature-Type");
        String wechatPayCertificateSerialNumber = request.getHeader("Wechatpay-Serial");
        requestParam = new RequestParam.Builder()
                .serialNumber(wechatPayCertificateSerialNumber)
                .nonce(nonce)
                .signature(signature)
                .timestamp(timestamp)
                .signType(singType)
                .body(body)
                .build();
        System.out.println(requestParam.toString());
        System.out.println("---------------------------信息获取完毕-------------------------------");
        return notificationParser;
    }
}
